@@ -109,12 +109,6 @@ clutter_offscreen_effect_set_actor (ClutterActorMeta *meta,
       priv->offscreen = COGL_INVALID_HANDLE;
     }
 
-  if (priv->target != COGL_INVALID_HANDLE)
-    {
-      cogl_handle_unref (priv->target);
-      priv->target = COGL_INVALID_HANDLE;
-    }
-
   /* we keep a back pointer here, to avoid going through the ActorMeta */
   priv->actor = clutter_actor_meta_get_actor (meta);
 }
@@ -155,16 +149,12 @@ update_fbo (ClutterEffect *effect)
    */
   clutter_actor_get_transformed_size (priv->actor, &width, &height);
   if (priv->target_width == width &&
-      priv->target_height == height)
+      priv->target_height == height &&
+      priv->offscreen != COGL_INVALID_HANDLE)
     return TRUE;
 
-  if (priv->target != COGL_INVALID_HANDLE)
-    {
-      cogl_handle_unref (priv->target);
-      cogl_handle_unref (priv->offscreen);
-    }
-
-  priv->target = cogl_material_new ();
+  if (priv->target == COGL_INVALID_HANDLE)
+    priv->target = cogl_material_new ();
 
   texture = clutter_offscreen_effect_create_texture (self, width, height);
   if (texture == COGL_INVALID_HANDLE)
@@ -180,6 +170,9 @@ update_fbo (ClutterEffect *effect)
   priv->target_width = cogl_texture_get_width (texture);
   priv->target_height = cogl_texture_get_height (texture);
 
+  if (priv->offscreen != COGL_INVALID_HANDLE)
+    cogl_handle_unref (priv->offscreen);
+
   priv->offscreen = cogl_offscreen_new_to_texture (texture);
   if (priv->offscreen == COGL_INVALID_HANDLE)
     {
