@@ -320,13 +320,15 @@ object_info_free (gpointer data)
         {
           if (oinfo->is_unmerged)
             {
-              /* destroy every actor, unless it's the default stage */
-              if (!oinfo->is_toplevel &&
-	          oinfo->is_stage_default == FALSE &&
-                  CLUTTER_IS_ACTOR (oinfo->object))
-                clutter_actor_destroy (CLUTTER_ACTOR (oinfo->object));
-
-              g_object_unref(oinfo->object);
+              if (oinfo->is_toplevel)
+                g_object_unref (oinfo->object);
+              else
+                {
+                  /* destroy every actor, unless it's the default stage */
+                  if (oinfo->is_stage_default == FALSE &&
+                      CLUTTER_IS_ACTOR (oinfo->object))
+                    clutter_actor_destroy (CLUTTER_ACTOR (oinfo->object));
+                }
             }
           else
             {
@@ -569,7 +571,7 @@ clutter_script_load_from_data (ClutterScript  *script,
  * Retrieves the object bound to @name. This function does not increment
  * the reference count of the returned object.
  *
- * Return value: : (transfer none): the named object, or %NULL if no object
+ * Return value: (transfer none): the named object, or %NULL if no object
  *   with the given name was available
  *
  * Since: 0.6
