@@ -409,6 +409,12 @@ clutter_stage_realize (ClutterActor *self)
    */
   if (is_realized)
     {
+     //MAEMO5 PR-1.3 FIX.//
+     CLUTTER_ACTOR_SET_FLAGS (self, CLUTTER_ACTOR_REALIZED);
+     clutter_stage_ensure_current (CLUTTER_STAGE (self));
+     return;
+     //MAEMO5 PR-1.3 FIX.//
+
       ClutterBackend *backend = clutter_get_default_backend ();
       GError *error = NULL;
 
