@@ -37,6 +37,7 @@
 #include "cogl-util.h"
 #include "cogl-context.h"
 #include "cogl-material-private.h"
+#include "cogl-material-opengl-private.h"
 #include "cogl-winsys.h"
 #include "cogl-framebuffer-private.h"
 #include "cogl-matrix-private.h"
@@ -266,10 +267,6 @@ _cogl_enable (unsigned long flags)
   */
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  toggle_flag (ctx, flags,
-               COGL_ENABLE_BLEND,
-               GL_BLEND);
-
   toggle_flag (ctx, flags,
                COGL_ENABLE_BACKFACE_CULLING,
                GL_CULL_FACE);
@@ -291,25 +288,28 @@ _cogl_get_enable (void)
   return ctx->enable_flags;
 }
 
+/* XXX: This API has been deprecated */
 void
 cogl_set_depth_test_enabled (gboolean setting)
 {
-  /* Currently the journal can't track changes to depth state... */
-  _cogl_journal_flush ();
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  if (setting)
-    {
-      glEnable (GL_DEPTH_TEST);
-      glDepthFunc (GL_LEQUAL);
-    }
+  if (ctx->legacy_depth_test_enabled == setting)
+    return;
+
+  ctx->legacy_depth_test_enabled = setting;
+  if (ctx->legacy_depth_test_enabled)
+    ctx->legacy_state_set++;
   else
-    glDisable (GL_DEPTH_TEST);
+    ctx->legacy_state_set--;
 }
 
+/* XXX: This API has been deprecated */
 gboolean
 cogl_get_depth_test_enabled (void)
 {
-  return glIsEnabled (GL_DEPTH_TEST) == GL_TRUE ? TRUE : FALSE;
+  _COGL_GET_CONTEXT (ctx, FALSE);
+  return ctx->legacy_depth_test_enabled;
 }
 
 void
@@ -373,9 +373,6 @@ cogl_set_source_color (const CoglColor *color)
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  /* In case cogl_set_source_texture was previously used... */
-  cogl_material_remove_layer (ctx->simple_material, 0);
-
   premultiplied = *color;
   cogl_color_premultiply (&premultiplied);
   cogl_material_set_color (ctx->simple_material, &premultiplied);
@@ -493,6 +490,15 @@ cogl_get_features (void)
   if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_VBOS))
     ctx->feature_flags &= ~COGL_FEATURE_VBOS;
 
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_PBOS))
+    ctx->feature_flags &= ~COGL_FEATURE_PBOS;
+
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_ARBFP))
+    ctx->feature_flags &= ~COGL_FEATURE_SHADERS_ARBFP;
+
+  if (G_UNLIKELY (cogl_debug_flags & COGL_DEBUG_DISABLE_GLSL))
+    ctx->feature_flags &= ~COGL_FEATURE_SHADERS_GLSL;
+
   return ctx->feature_flags;
 }
 
@@ -507,6 +513,17 @@ cogl_features_available (CoglFeatureFlags features)
   return (ctx->feature_flags & features) == features;
 }
 
+gboolean
+_cogl_features_available_private (CoglFeatureFlagsPrivate features)
+{
+  _COGL_GET_CONTEXT (ctx, 0);
+
+  if (!ctx->features_cached)
+    _cogl_features_init ();
+
+  return (ctx->feature_flags_private & features) == features;
+}
+
 /* XXX: This function should either be replaced with one returning
  * integers, or removed/deprecated and make the
  * _cogl_framebuffer_get_viewport* functions public.
@@ -533,21 +550,21 @@ cogl_get_bitmasks (int *red,
                    int *blue,
                    int *alpha)
 {
-  CoglHandle framebuffer;
+  CoglFramebuffer *framebuffer;
 
   framebuffer = _cogl_get_framebuffer ();
 
   if (red)
-    *red = cogl_framebuffer_get_red_bits (framebuffer);
+    *red = _cogl_framebuffer_get_red_bits (framebuffer);
 
   if (green)
-    *green = cogl_framebuffer_get_green_bits (framebuffer);
+    *green = _cogl_framebuffer_get_green_bits (framebuffer);
 
   if (blue)
-    *blue = cogl_framebuffer_get_blue_bits (framebuffer);
+    *blue = _cogl_framebuffer_get_blue_bits (framebuffer);
 
   if (alpha)
-    *alpha = cogl_framebuffer_get_alpha_bits (framebuffer);
+    *alpha = _cogl_framebuffer_get_alpha_bits (framebuffer);
 }
 
 void
@@ -557,53 +574,28 @@ cogl_set_fog (const CoglColor *fog_color,
               float            z_near,
               float            z_far)
 {
-  GLfloat fogColor[4];
-  GLenum gl_mode = GL_LINEAR;
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  /* The cogl journal doesn't currently track fog state changes */
-  _cogl_journal_flush ();
+  if (ctx->legacy_fog_state.enabled == FALSE)
+    ctx->legacy_state_set++;
 
-  fogColor[0] = cogl_color_get_red_float (fog_color);
-  fogColor[1] = cogl_color_get_green_float (fog_color);
-  fogColor[2] = cogl_color_get_blue_float (fog_color);
-  fogColor[3] = cogl_color_get_alpha_float (fog_color);
-
-  glEnable (GL_FOG);
-
-  glFogfv (GL_FOG_COLOR, fogColor);
-
-#if HAVE_COGL_GLES
-  switch (mode)
-    {
-    case COGL_FOG_MODE_LINEAR:
-      gl_mode = GL_LINEAR;
-      break;
-    case COGL_FOG_MODE_EXPONENTIAL:
-      gl_mode = GL_EXP;
-      break;
-    case COGL_FOG_MODE_EXPONENTIAL_SQUARED:
-      gl_mode = GL_EXP2;
-      break;
-    }
-#endif
-  /* TODO: support other modes for GLES2 */
-
-  /* NB: GLES doesn't have glFogi */
-  glFogf (GL_FOG_MODE, gl_mode);
-  glHint (GL_FOG_HINT, GL_NICEST);
-
-  glFogf (GL_FOG_DENSITY, (GLfloat) density);
-  glFogf (GL_FOG_START, (GLfloat) z_near);
-  glFogf (GL_FOG_END, (GLfloat) z_far);
+  ctx->legacy_fog_state.enabled = TRUE;
+  ctx->legacy_fog_state.color = *fog_color;
+  ctx->legacy_fog_state.mode = mode;
+  ctx->legacy_fog_state.density = density;
+  ctx->legacy_fog_state.z_near = z_near;
+  ctx->legacy_fog_state.z_far = z_far;
 }
 
 void
 cogl_disable_fog (void)
 {
-  /* Currently the journal can't track changes to fog state... */
-  _cogl_journal_flush ();
+  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
-  glDisable (GL_FOG);
+  if (ctx->legacy_fog_state.enabled == TRUE)
+    ctx->legacy_state_set--;
+
+  ctx->legacy_fog_state.enabled = FALSE;
 }
 
 void
@@ -622,12 +614,14 @@ cogl_read_pixels (int x,
                   guint8 *pixels)
 {
   CoglFramebuffer *framebuffer;
-  int        framebuffer_height;
-  int        bpp;
-  CoglBitmap bmp;
-  GLenum     gl_intformat;
-  GLenum     gl_format;
-  GLenum     gl_type;
+  int              framebuffer_height;
+  int              bpp;
+  CoglBitmap      *bmp;
+  GLenum           gl_intformat;
+  GLenum           gl_format;
+  GLenum           gl_type;
+  CoglPixelFormat  bmp_format;
+  int              rowstride;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
@@ -654,11 +648,8 @@ cogl_read_pixels (int x,
 
   /* Initialise the CoglBitmap */
   bpp = _cogl_get_format_bpp (format);
-  bmp.format = format;
-  bmp.data = pixels;
-  bmp.width = width;
-  bmp.height = height;
-  bmp.rowstride = bpp * width;
+  rowstride = bpp * width;
+  bmp_format = format;
 
   if ((format & COGL_A_BIT))
     {
@@ -672,9 +663,13 @@ cogl_read_pixels (int x,
          it. Eventually we may want to add a way for an application to
          inform Cogl that the framebuffer is not premultiplied in case
          it is being used for some special purpose. */
-      bmp.format |= COGL_PREMULT_BIT;
+      bmp_format |= COGL_PREMULT_BIT;
     }
 
+  bmp = _cogl_bitmap_new_from_data (pixels,
+                                    bmp_format, width, height, rowstride,
+                                    NULL, NULL);
+
   _cogl_pixel_format_to_gl (format, &gl_intformat, &gl_format, &gl_type);
 
   /* Under GLES only GL_RGBA with GL_UNSIGNED_BYTE as well as an
@@ -688,33 +683,33 @@ cogl_read_pixels (int x,
 #ifndef COGL_HAS_GL
   if (gl_format != GL_RGBA || gl_type != GL_UNSIGNED_BYTE)
     {
-      CoglBitmap tmp_bmp, dst_bmp;
+      CoglBitmap *tmp_bmp, *dst_bmp;
+      guint8 *tmp_data = g_malloc (width * height * 4);
 
-      tmp_bmp.format = COGL_PIXEL_FORMAT_RGBA_8888_PRE;
-      tmp_bmp.data = g_malloc (width * height * 4);
-      tmp_bmp.width = width;
-      tmp_bmp.height = height;
-      tmp_bmp.rowstride = 4 * width;
+      tmp_bmp = _cogl_bitmap_new_from_data (tmp_data,
+                                            COGL_PIXEL_FORMAT_RGBA_8888_PRE,
+                                            width, height, 4 * width,
+                                            (CoglBitmapDestroyNotify) g_free,
+                                            NULL);
 
-      _cogl_texture_driver_prep_gl_for_pixels_download (tmp_bmp.rowstride, 4);
+      _cogl_texture_driver_prep_gl_for_pixels_download (4 * width, 4);
 
       GE( glReadPixels (x, y, width, height,
                         GL_RGBA, GL_UNSIGNED_BYTE,
-                        tmp_bmp.data) );
+                        tmp_data) );
 
       /* CoglBitmap doesn't currently have a way to convert without
          allocating its own buffer so we have to copy the data
          again */
-      if (_cogl_bitmap_convert_format_and_premult (&tmp_bmp,
-                                                   &dst_bmp,
-                                                   bmp.format))
+      if ((dst_bmp = _cogl_bitmap_convert_format_and_premult (tmp_bmp,
+                                                              bmp_format)))
         {
-          _cogl_bitmap_copy_subregion (&dst_bmp,
-                                       &bmp,
+          _cogl_bitmap_copy_subregion (dst_bmp,
+                                       bmp,
                                        0, 0,
                                        0, 0,
-                                       bmp.width, bmp.height);
-          g_free (dst_bmp.data);
+                                       width, height);
+          cogl_object_unref (dst_bmp);
         }
       else
         {
@@ -722,26 +717,26 @@ cogl_read_pixels (int x,
              just have to leave the data initialised */
         }
 
-      g_free (tmp_bmp.data);
+      cogl_object_unref (tmp_bmp);
     }
   else
 #endif
     {
-      _cogl_texture_driver_prep_gl_for_pixels_download (bmp.rowstride, bpp);
+      _cogl_texture_driver_prep_gl_for_pixels_download (rowstride, bpp);
 
       GE( glReadPixels (x, y, width, height, gl_format, gl_type, pixels) );
 
       /* Convert to the premult format specified by the caller
          in-place. This will do nothing if the premult status is already
          correct. */
-      _cogl_bitmap_convert_premult_status (&bmp, format);
+      _cogl_bitmap_convert_premult_status (bmp, format);
     }
 
   /* NB: All offscreen rendering is done upside down so there is no need
    * to flip in this case... */
   if (!cogl_is_offscreen (framebuffer))
     {
-      guint8 *temprow = g_alloca (bmp.rowstride * sizeof (guint8));
+      guint8 *temprow = g_alloca (rowstride * sizeof (guint8));
 
       /* TODO: consider using the GL_MESA_pack_invert extension in the future
        * to avoid this flip... */
@@ -752,15 +747,17 @@ cogl_read_pixels (int x,
           if (y != height - y - 1) /* skip center row */
             {
               memcpy (temprow,
-                      pixels + y * bmp.rowstride, bmp.rowstride);
-              memcpy (pixels + y * bmp.rowstride,
-                      pixels + (height - y - 1) * bmp.rowstride, bmp.rowstride);
-              memcpy (pixels + (height - y - 1) * bmp.rowstride,
+                      pixels + y * rowstride, rowstride);
+              memcpy (pixels + y * rowstride,
+                      pixels + (height - y - 1) * rowstride, rowstride);
+              memcpy (pixels + (height - y - 1) * rowstride,
                       temprow,
-                      bmp.rowstride);
+                      rowstride);
             }
         }
     }
+
+  cogl_object_unref (bmp);
 }
 
 static void
@@ -796,7 +793,6 @@ _cogl_disable_other_texcoord_arrays (const CoglBitmask *mask)
 void
 cogl_begin_gl (void)
 {
-  CoglMaterialFlushOptions options;
   unsigned long enable_flags = 0;
 
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
@@ -831,12 +827,7 @@ cogl_begin_gl (void)
    * A user should instead call cogl_set_source_color4ub() before
    * cogl_begin_gl() to simplify the state flushed.
    */
-  options.flags = 0;
-  _cogl_material_flush_gl_state (ctx->source_material, &options);
-
-  /* FIXME: This api is a bit yukky, ideally it will be removed if we
-   * re-work the _cogl_enable mechanism */
-  enable_flags |= _cogl_material_get_cogl_enable_flags (ctx->source_material);
+  _cogl_material_flush_gl_state (ctx->source_material, FALSE);
 
   if (ctx->enable_backface_culling)
     enable_flags |= COGL_ENABLE_BACKFACE_CULLING;
@@ -865,102 +856,6 @@ cogl_end_gl (void)
   ctx->in_begin_gl_block = FALSE;
 }
 
-static CoglTextureUnit *
-_cogl_texture_unit_new (int index_)
-{
-  CoglTextureUnit *unit = g_new0 (CoglTextureUnit, 1);
-  unit->matrix_stack = _cogl_matrix_stack_new ();
-  unit->index = index_;
-  return unit;
-}
-
-static void
-_cogl_texture_unit_free (CoglTextureUnit *unit)
-{
-  _cogl_matrix_stack_destroy (unit->matrix_stack);
-  g_free (unit);
-}
-
-CoglTextureUnit *
-_cogl_get_texture_unit (int index_)
-{
-  GList *l;
-  CoglTextureUnit *unit;
-
-  _COGL_GET_CONTEXT (ctx, NULL);
-
-  for (l = ctx->texture_units; l; l = l->next)
-    {
-      unit = l->data;
-
-      if (unit->index == index_)
-        return unit;
-
-      /* The units are always sorted, so at this point we know this unit
-       * doesn't exist */
-      if (unit->index > index_)
-        break;
-    }
-  /* NB: if we now insert a new layer before l, that will maintain order.
-   */
-
-  unit = _cogl_texture_unit_new (index_);
-
-  /* Note: see comment after for() loop above */
-  ctx->texture_units =
-    g_list_insert_before (ctx->texture_units, l, unit);
-
-  return unit;
-}
-
-void
-_cogl_destroy_texture_units (void)
-{
-  GList *l;
-
-  _COGL_GET_CONTEXT (ctx, NO_RETVAL);
-
-  for (l = ctx->texture_units; l; l = l->next)
-    _cogl_texture_unit_free (l->data);
-  g_list_free (ctx->texture_units);
-}
-
-/*
- * This is more complicated than that, another pass needs to be done when
- * cogl have a neat way of saying if we are using the fixed function pipeline
- * or not (for the GL case):
- * MAX_TEXTURE_UNITS: fixed function pipeline, a texture unit has both a
- *                    sampler and a set of texture coordinates
- * MAX_TEXTURE_IMAGE_UNITS: number of samplers one can use from a fragment
- *                          program/shader (ARBfp1.0 asm/GLSL)
- * MAX_VERTEX_TEXTURE_UNITS: number of samplers one can use from a vertex
- *                           program/shader (can be 0)
- * MAX_COMBINED_TEXTURE_IMAGE_UNITS: Maximum samplers one can use, counting both
- *                                   the vertex and fragment shaders
- *
- * If both the vertex shader and the fragment processing stage access the same
- * texture image unit, then that counts as using two texture image units
- * against the latter limit: http://www.opengl.org/sdk/docs/man/xhtml/glGet.xml
- *
- * Note that, for now, we use GL_MAX_TEXTURE_UNITS as we are exposing the
- * fixed function pipeline.
- */
-unsigned int
-_cogl_get_max_texture_image_units (void)
-{
-  _COGL_GET_CONTEXT (ctx, 0);
-
-  /* This function is called quite often so we cache the value to
-     avoid too many GL calls */
-  if (ctx->max_texture_units == -1)
-    {
-      ctx->max_texture_units = 1;
-      GE( glGetIntegerv (GL_MAX_TEXTURE_UNITS, &ctx->max_texture_units) );
-    }
-
-  return ctx->max_texture_units;
-}
-
 void
 cogl_push_matrix (void)
 {
@@ -1143,16 +1038,12 @@ cogl_set_source (CoglHandle material_handle)
 void
 cogl_set_source_texture (CoglHandle texture_handle)
 {
-  CoglColor white;
-
   _COGL_GET_CONTEXT (ctx, NO_RETVAL);
 
   g_return_if_fail (texture_handle != NULL);
 
-  cogl_material_set_layer (ctx->simple_material, 0, texture_handle);
-  cogl_color_set_from_4ub (&white, 0xff, 0xff, 0xff, 0xff);
-  cogl_material_set_color (ctx->simple_material, &white);
-  cogl_set_source (ctx->simple_material);
+  cogl_material_set_layer (ctx->texture_material, 0, texture_handle);
+  cogl_set_source (ctx->texture_material);
 }
 
 void
@@ -1163,7 +1054,7 @@ cogl_set_source_color4ub (guint8 red,
 {
   CoglColor c = { 0, };
 
-  cogl_color_set_from_4ub (&c, red, green, blue, alpha);
+  cogl_color_init_from_4ub (&c, red, green, blue, alpha);
   cogl_set_source_color (&c);
 }
 
@@ -1175,7 +1066,53 @@ cogl_set_source_color4f (float red,
 {
   CoglColor c = { 0, };
 
-  cogl_color_set_from_4f (&c, red, green, blue, alpha);
+  cogl_color_init_from_4f (&c, red, green, blue, alpha);
   cogl_set_source_color (&c);
 }
 
+/* Scale from OpenGL normalized device coordinates (ranging from -1 to 1)
+ * to Cogl window/framebuffer coordinates (ranging from 0 to buffer-size) with
+ * (0,0) being top left. */
+#define VIEWPORT_TRANSFORM_X(x, vp_origin_x, vp_width) \
+    (  ( ((x) + 1.0) * ((vp_width) / 2.0) ) + (vp_origin_x)  )
+/* Note: for Y we first flip all coordinates around the X axis while in
+ * normalized device coodinates */
+#define VIEWPORT_TRANSFORM_Y(y, vp_origin_y, vp_height) \
+    (  ( ((-(y)) + 1.0) * ((vp_height) / 2.0) ) + (vp_origin_y)  )
+
+/* Transform a homogeneous vertex position from model space to Cogl
+ * window coordinates (with 0,0 being top left) */
+void
+_cogl_transform_point (const CoglMatrix *matrix_mv,
+                       const CoglMatrix *matrix_p,
+                       const float *viewport,
+                       float *x,
+                       float *y)
+{
+  float z = 0;
+  float w = 1;
+
+  /* Apply the modelview matrix transform */
+  cogl_matrix_transform_point (matrix_mv, x, y, &z, &w);
+
+  /* Apply the projection matrix transform */
+  cogl_matrix_transform_point (matrix_p, x, y, &z, &w);
+
+  /* Perform perspective division */
+  *x /= w;
+  *y /= w;
+
+  /* Apply viewport transform */
+  *x = VIEWPORT_TRANSFORM_X (*x, viewport[0], viewport[2]);
+  *y = VIEWPORT_TRANSFORM_Y (*y, viewport[1], viewport[3]);
+}
+
+#undef VIEWPORT_TRANSFORM_X
+#undef VIEWPORT_TRANSFORM_Y
+
+GQuark
+_cogl_error_quark (void)
+{
+  return g_quark_from_static_string ("cogl-error-quark");
+}
+
