@@ -132,7 +132,6 @@ clutter_backend_egl_post_parse (ClutterBackend  *backend,
   return TRUE;
 }
 
-static gboolean first_run=TRUE;
 
 static gboolean
 clutter_backend_egl_create_context (ClutterBackend  *backend,
@@ -348,7 +347,6 @@ retry:
    * current immediately to make sure the Cogl and Clutter
    * can query the EGL context for features.
    */
-if (first_run) first_run=FALSE, //MAEMO5
 
   status = eglMakeCurrent (backend_egl->edpy,
                            backend_egl->egl_surface,
