@@ -108,10 +108,13 @@ xsettings_filter (XEvent       *xevent,
 {
   ClutterBackendX11 *backend_x11 = data;
 
-  if (_clutter_xsettings_client_process_event (backend_x11->xsettings, xevent))
-    return CLUTTER_X11_FILTER_REMOVE;
-  else
-    return CLUTTER_X11_FILTER_CONTINUE;
+  _clutter_xsettings_client_process_event (backend_x11->xsettings, xevent);
+
+  /* we always want the rest of the stack to get XSettings events, even
+   * if Clutter already handled them
+   */
+
+  return CLUTTER_X11_FILTER_CONTINUE;
 }
 
 static Bool
@@ -158,6 +161,9 @@ clutter_backend_x11_xsettings_notify (const char       *name,
   if (name == NULL || *name == '\0')
     return;
 
+  if (setting == NULL)
+    return;
+
   g_object_freeze_notify (G_OBJECT (settings));
 
   for (i = 0; i < _n_clutter_settings_map; i++)
@@ -316,6 +322,12 @@ clutter_backend_x11_post_parse (ClutterBackend  *backend,
                       "backend", backend_x11,
                       NULL);
 
+      /* register keymap */
+      backend_x11->keymap =
+        g_object_new (CLUTTER_TYPE_KEYMAP_X11,
+                      "backend", backend_x11,
+                      NULL);
+
       /* create XSETTINGS client */
       backend_x11->xsettings =
         _clutter_xsettings_client_new (backend_x11->xdpy,
@@ -472,6 +484,29 @@ clutter_backend_x11_get_features (ClutterBackend *backend)
   return CLUTTER_FEATURE_STAGE_USER_RESIZE | CLUTTER_FEATURE_STAGE_CURSOR;
 }
 
+static void
+clutter_backend_x11_copy_event_data (ClutterBackend     *backend,
+                                     const ClutterEvent *src,
+                                     ClutterEvent       *dest)
+{
+  gpointer event_x11;
+
+  event_x11 = _clutter_event_get_platform_data (src);
+  if (event_x11 != NULL)
+    _clutter_event_set_platform_data (dest, _clutter_event_x11_copy (event_x11));
+}
+
+static void
+clutter_backend_x11_free_event_data (ClutterBackend *backend,
+                                     ClutterEvent   *event)
+{
+  gpointer event_x11;
+
+  event_x11 = _clutter_event_get_platform_data (event);
+  if (event_x11 != NULL)
+    _clutter_event_x11_free (event_x11);
+}
+
 gboolean
 clutter_backend_x11_handle_event (ClutterBackendX11 *backend_x11,
                                   XEvent *xevent)
@@ -513,6 +548,8 @@ clutter_backend_x11_class_init (ClutterBackendX11Class *klass)
   backend_class->add_options = clutter_backend_x11_add_options;
   backend_class->get_features = clutter_backend_x11_get_features;
   backend_class->get_device_manager = clutter_backend_x11_get_device_manager;
+  backend_class->copy_event_data = clutter_backend_x11_copy_event_data;
+  backend_class->free_event_data = clutter_backend_x11_free_event_data;
 
   backendx11_class->handle_event = clutter_backend_x11_handle_event;
 }
@@ -639,12 +676,27 @@ clutter_x11_enable_xinput (void)
 }
 
 /**
- * clutter_x11_disable_event_retrieval
+ * clutter_x11_disable_event_retrieval:
  *
- * Disables retrieval of X events in the main loop. Use to create event-less
- * canvas or in conjunction with clutter_x11_handle_event.
+ * Disables the internal polling of X11 events in the main loop.
  *
- * This function can only be called before calling clutter_init().
+ * Libraries or applications calling this function will be responsible of
+ * polling all X11 events.
+ *
+ * You also must call clutter_x11_handle_event() to let Clutter process
+ * events and maintain its internal state.
+ *
+ * <warning>This function can only be called before calling
+ * clutter_init().</warning>
+ *
+ * <note>Even with event handling disabled, Clutter will still select
+ * all the events required to maintain its internal state on the stage
+ * Window; compositors using Clutter and input regions to pass events
+ * through to application windows should not rely on an empty input
+ * region, and should instead clear it themselves explicitly using the
+ * XFixes extension.</note>
+ *
+ * This function should not be normally used by applications.
  *
  * Since: 0.8
  */
@@ -662,7 +714,7 @@ clutter_x11_disable_event_retrieval (void)
 }
 
 /**
- * clutter_x11_has_event_retrieval
+ * clutter_x11_has_event_retrieval:
  *
  * Queries the X11 backend to check if event collection has been disabled.
  *
@@ -939,7 +991,7 @@ clutter_x11_has_composite_extension (void)
 void
 clutter_x11_set_use_argb_visual (gboolean use_argb)
 {
-  if (backend_singleton != NULL)
+  if (_clutter_context_is_initialized ())
     {
       g_warning ("%s() can only be used before calling clutter_init()",
                  G_STRFUNC);
